import kombu

from .errors import AlreadyRegistered
from .registry import _exchange_registry, exchange_exists


def register_exchange(app_provider, name, fail_silently: bool = False, _type='direct'):
    if exchange_exists(name):
        if not fail_silently:
            raise AlreadyRegistered(f'exchange with name {name} already registered')
        else:
            return

    connection = app_provider.get_connection()
    exchange = kombu.Exchange(
        name=name,
        type=_type,
        durable=True,
        channel=connection
    )

    exchange.declare()

    _exchange_registry[name] = exchange



from .errors import NotRegistered

__all__ = (
    'get_exchange',
    'get_queue',
    '_exchange_registry',
    '_queue_registry',
    'queue_exists',
    'exchange_exists'
)

_exchange_registry = {}
_queue_registry = {}


def queue_exists(name):
    return _queue_registry.get(name, None) is not None


def get_queue(name, routing_key):
    queue_name = f'{name}_{routing_key}'
    queue = _queue_registry.get(queue_name, None)
    if not queue:
        raise NotRegistered(
            """
            `%s` has not been registered in the queue registry. has it been imported?
            """.strip()
            % name
        )
    return queue


def exchange_exists(name):
    return _exchange_registry.get(name, None) is not None


def get_exchange(name):
    exchange = _exchange_registry.get(name, None)
    if not exchange:
        raise NotRegistered(
            """
            `%s` has not been registered in the exchange registry. has it been imported?
            """.strip()
            % name
        )
    return exchange



import os

from .errors import ImproperlyConfigured


def get_environment(key, fail_silently=False, default=None):
    """ Get the environment variable or return exception """

    try:
        return os.environ[key]
    except KeyError:
        if not fail_silently:
            raise ImproperlyConfigured
        else:
            return default



from __future__ import absolute_import

from celery import Celery

from .settings import DEFAULT_APP_NAME, DEFAULT_BROKER_URL, \
    INSTALLED_APPS, DEFAULT_CONFIG_SETTINGS, DEFAULT_NAMESPACE


class CeleryAppProvider:

    def __init__(self, app_name=None, namespace=None, broker_url=None,
                 installed_apps=None, task_autodiscover=True,
                 object_config=None):
        if not app_name:
            app_name = DEFAULT_APP_NAME
        if not broker_url:
            broker_url = DEFAULT_BROKER_URL
        if not installed_apps:
            installed_apps = INSTALLED_APPS
        if not object_config:
            object_config = DEFAULT_CONFIG_SETTINGS
        if not namespace:
            namespace = DEFAULT_NAMESPACE
        self.app = Celery(app_name, broker=broker_url)
        self.app.config_from_object(object_config, namespace=namespace)
        if task_autodiscover:
            self.app.autodiscover_tasks(installed_apps)

    def get_connection(self, block: bool = True):
        return self.app.pool.acquire(block=block)

    def get_producer(self, block: bool = True):
        return self.app.producer_pool.acquire(block=block)

    def get_app(self):
        return self.app

    def add_tasks(self, task):
        # future plan
        pass

    def add_consumer(self, consumer):
        self.app.steps['consumer'].add(consumer)



from .utils import get_environment

DEFAULT_APP_NAME = 'CELERY-RMQ'
DEFAULT_NAMESPACE = 'CELERY'

# don't use default installed_apps, send installed_apps list
# while creating instance of the CeleryAppProvider
# because default installed apps list is empty
# so task auto discover won't work

INSTALLED_APPS = [

]

# Rabbit MQ and Celery Broker Configuration for task queues
# EMAIL Sending Task queues requires task managers
# Rabbit MQ and Celery is here for task management

RABBIT_MQ_USER = get_environment('RABBIT_MQ_USER', default='admin', fail_silently=True)
RABBIT_MQ_PASSWORD = get_environment('RABBIT_MQ_PASSWORD', default='admin', fail_silently=True)
BROKER_HOST = get_environment('CRMQ_BROKER_HOST', default='localhost', fail_silently=True)
BROKER_PORT = get_environment('CRMQ_BROKER_PORT', default='5672', fail_silently=True)

DEFAULT_BROKER_URL = f'amqp://{RABBIT_MQ_USER}:{RABBIT_MQ_PASSWORD}@{BROKER_HOST}:{BROKER_PORT}/'

DEFAULT_CONFIG_SETTINGS = 'celery_rmq.settings'

# DEFAULT config for celery config from object

CELERY_ACCEPT_CONTENT = ['json']
CELERY_RESULT_SERIALIZER = 'json'
CELERY_TASK_SERIALIZER = 'json'



import kombu

from .registry import get_exchange, _queue_registry


def register_queue(app_provider, name, routing_key, exchange_name, queue_arguments=None):
    exchange = get_exchange(exchange_name)
    connection = app_provider.get_connection()

    if not queue_arguments:
        queue_arguments = {'x-queue-type': 'classic'}

    queue = kombu.Queue(
        name=name,
        exchange=exchange,
        routing_key=routing_key,
        channel=connection,
        message_ttl=600,
        queue_arguments=queue_arguments,
        durable=True
    )

    queue.declare()

    _queue_registry[f'{name}_{routing_key}'] = queue



import setuptools

NAME = "celery-rmq"
DESCRIPTION = "Celery and RabbitMQ producer and consumer made easy with celery-rmq"
URL = "https://github.com/knroy/celery-rmq"
EMAIL = "rax.komol@gmail.com   "
AUTHOR = "Komol Nath Roy"
REQUIRES_PYTHON = ">=3.6.0"
VERSION = "0.0.1"

# What packages are required for this module to be executed?

REQUIRED = [
    "celery==4.4.2",
    "kombu==4.6.8"
]

# What packages are optional?

EXTRAS = {

}

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!

try:
    with open("README.md", "r") as fh:
        long_description = fh.read()
except FileNotFoundError:
    long_description = DESCRIPTION

setuptools.setup(
    name=NAME,
    version=VERSION,
    author=AUTHOR,
    author_email=EMAIL,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=URL,
    packages=setuptools.find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    install_requires=REQUIRED,
    include_package_data=True,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Operating System :: OS Independent",
    ],
    python_requires=REQUIRES_PYTHON
)



__all__ = ['NotRegistered', 'AlreadyRegistered', 'ImproperlyConfigured']


class NotRegistered(Exception):
    pass


class AlreadyRegistered(Exception):
    pass


class ImproperlyConfigured(Exception):
    pass
